<?php
/**
    * Plugin Name:novicashtransfer Payment Gateway
    * Plugin URI :https://www.novicash.cash/
    * Description:novicashtransfer payment gateway for Woocommerce
    * Version:1.0.0
    * Author:novicashtransfer developer Team
    * Author URI:https://www.novicash.cash/
    * License:GPL2
    * License URI:https://www.gnu.org/licenses/gpl-2.0.html
    * Text Domain:woocommerce_novicashtransfer
    * Domain Path:/languages
*/

//Load all rest api classes

use Novicashtransfer\Api\Payer;
use Novicashtransfer\Api\Amount;
use Novicashtransfer\Api\Transaction;
use Novicashtransfer\Api\RedirectUrls;
use Novicashtransfer\Api\Payment;


add_action( 'plugins_loaded', 'woocommerceNovicashtransferPaymentGateway' );

add_action( 'init', 'checkForWoocommerceNovicashtransfer' );

function checkForWoocommerceNovicashtransfer() 
{
    if( isset( $_GET['woocommercenovicashtransfer'] ) ) {
        WC()->payment_gateways(); 
        do_action( 'checkwoocommercenovicashtransfer' );
    }
    
}

/**
 * Add Gateway class to all payment gateway methods
*/

function woocommerceAddNovicashtransferPayGatewayClass( $methods ) {
    
    $methods[] = 'WoocommerceNovicashtransferGateway'; 
    return $methods;
}

add_filter( 'woocommerce_payment_gateways', 'woocommerceAddNovicashtransferPayGatewayClass' );

// Loaded the plugin after installed

function woocommerceNovicashtransferPaymentGateway() 
{
    class WoocommerceNovicashtransferGateway extends WC_Payment_Gateway 
    {
        /**
         * Constructor for your shipping class
         *
         * @access public
         * @return void
         */
        public function __construct() 
        {
            $this->id                   = 'woocommercenovicashtransferpaymentgatewayid'; 
            $this->method_title         = __( 'Novicashtransfer', 'woocommerce_novicashtransfer_payment' );  
            $this->method_description   = __( 'WooCommerce Payment Gateway for Novicashtransfer', 'woocommercenovicashtransferpaymentgatewayid' );
            $this->title                = __( 'Novicashtransfer', 'woocommercenovicashtransferpaymentgatewayid' );
            $this->has_fields           = false;
            $this->supports             = array('products');

            $this->getNovicashtransferSdk();

            // Load the settings.

            $this->initFormFields();
            $this->init_settings();

            //Get option from admin settings filled by admin

            $this->enabled              = $this->get_option('enabled');
            $this->title                = $this->get_option('title');
            $this->description          = $this->get_option('description');
            $this->client_id            = $this->get_option('client_id');
            $this->client_secret        = $this->get_option('client_secret');
            
            add_action( 'checkwoocommercenovicashtransfer', array( $this, 'checkResponse') );

            // Save settings
            if ( is_admin() ) {
                // Versions over 2.0
                // Save our administration options. Since we are not going to be doing anything special
                // we have not defined 'process_admin_options' in this class so the method in the parent
                // class will be used instead
                add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
            }   
        }

        /**
         * Load all payment api sdk
         *
         * @method initFormFields
         *
         */

        private function getNovicashtransferSdk() 
        {
            require_once plugin_dir_path( __FILE__ ) . 'includes/novicashtransfer/vendor/autoload.php';
        }

         /**
         * Load all payment method configuaration in admin area
         *
         * @method initFormFields
         *
         */

        public function initFormFields() 
        {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __( 'Enable', 'woocommercenovicashtransferpaymentgatewayid' ),
                    'type' => 'checkbox',
                    'label' => __( 'Enable Novicashtransfer', 'woocommercenovicashtransferpaymentgatewayid' ),
                    'default' => 'yes'
                ),
                'title' => array(
                        'title' => __( 'Title', 'woocommercenovicashtransferpaymentgatewayid' ),
                        'type' => 'text',
                        'description' => __('This controls the title which the user sees during checkout.','woocommercenovicashtransferpaymentgatewayid'),
                        'default' => 'Novicashtransfer'
                ),
                'description' => array(
                        'title'       => 'Description',
                        'type'        => 'textarea',
                        'description' => 'This controls the description which the user sees during checkout.',
                        'default'     => 'Pay with Novicashtransfer.',
                ),
                'client_id' => array(
                    'title' => __( 'Client ID', 'woocommercenovicashtransferpaymentgatewayid' ),
                    'type' => 'text',
                    'default' => ''
                ),
                'client_secret' => array(
                    'title' => __( 'Client Secret', 'woocommercenovicashtransferpaymentgatewayid' ),
                    'type' => 'password',
                    'default' => ''
                ),
            );
        }

        /**
         * Process payment in woocommerce dashboard after confirming payment
         *
         * @param $order_id
         *
         * @return approvedUrl
         */
        public function process_payment( $order_id ) 
        {
            global $woocommerce;
            $order       = new WC_Order( $order_id );
            $payer       = new Payer();
            $payer->setPaymentMethod("Novicashtransfer");
            $amount      = new Amount();
            $amount->setTotal( $order->get_total() )->setCurrency( get_woocommerce_currency() );
            $transaction = new Transaction();
            $transaction->setAmount($amount);
            $baseUrl     = $this->get_return_url( $order );
            if( strpos( $baseUrl, '?') !== false ) {
                $baseUrl .= '&';
            } else {
                $baseUrl .= '?';
            }
            $returnUrl    = $baseUrl . 'woocommercenovicashtransfer=true&order_id=' . $order_id;
            $redirectUrls = new RedirectUrls();
            $redirectUrls->setReturnUrl( $returnUrl )->setCancelUrl( $order->get_cancel_endpoint() );
            $payment      = new Payment();
            $payment->setCredentials([
                    'client_id' => $this->get_option('client_id'),
                    'client_secret' => $this->get_option('client_secret')
                ])
                ->setRedirectUrls($redirectUrls)
                ->setPayer($payer)
                ->setTransaction($transaction);
            try {
                $payment->create();
                $approvalUrl = $payment->getApprovedUrl();
                return array(
                    'result' => 'success',
                    'redirect' => $approvalUrl
                );
            } catch (Exception $ex) {
                wc_add_notice(  $ex->getMessage(), 'error' );
            }
            return array(
                    'result' => 'failure',
                    'redirect' => ''
            );
        }
        
        /**
         * Check reponse after complete the payment
         *
         * @param $woocommerce
         *
         * @return update order status
         */
        public function checkResponse() 
        {
            global $woocommerce;
             
            if (isset($_GET['woocommercenovicashtransfer'])) {

                $woocommercenovicashtransfer = $_GET['woocommercenovicashtransfer'];
                $orderId                   = explode('?', $_GET['order_id']);
                $order_id                  = $orderId[0];
                if ( $order_id == 0 || $order_id == '' ) {
                    return;
                }
                $order                = new WC_Order( $order_id );
                if ($order->has_status('completed') || $order->has_status('processing')) {
                    return;
                }
                //Get Payment method id
                $available_gateways = $woocommerce->payment_gateways->get_available_payment_gateways();
                $current_gateway    = '';

                if ( !empty( $available_gateways ) ) {
                    if ( isset( $woocommerce->session->chosen_payment_method ) && isset( $available_gateways[ $woocommerce->session->chosen_payment_method ] ) ) {
                        $current_gateway = $available_gateways[ $woocommerce->session->chosen_payment_method ];
                    } 
                }
                $paymentId = $current_gateway->id;

                if ($woocommercenovicashtransfer == true) {
                    // Payment complete
                    $order->update_status('completed');
                    // Add order note
                    $order->add_order_note( sprintf( __( '%s payment approved! Trnsaction ID: %s', 'woocommerce' ), $this->title, $order_id ) );
                    // Remove cart
                    $woocommerce->cart->empty_cart();
          
                }
                if ($woocommercenovicashtransfer == 'cancel') { 
                    $order = new WC_Order( $order_id );
                    $order->update_status('cancelled', sprintf( __( '%s payment cancelled! Transaction ID: %d', 'woocommerce' ), $this->title, $paymentId ) );
                }
            }
            return;
        }
    }

}

// ...